/*
 * $Id: cebook.cpp,v 1.3.2.2 2001/04/14 23:00:03 hige Exp $
 *eb
 * Copyright (C) 1999 SUZUKI, Mitsuhiro <hige@higesoft.rim.or.jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <strstream>
#include <iomanip>

#include "cebook.hpp"

using namespace ebook;

static string* g_font_path;

// Constants
static const int BUFSIZE                = 4096;
static const int MAX_HITS               = 64;
static const int BASE_FONT_SIZE         = EB_SIZE_NARROW_FONT_16;
static const EB_Font_Code BASE_FONT_TYPE = EB_FONT_16;
static const int SHOW_PROGRESS_COUNT    = 4;
static const char INSERT_MARK           = '\a';
static const char DIRECTORY_SEPARATOR   = '/';

#define EMPTY                             0

// Hook
#define HOOK_PROTOTYPE(hook)    \
        EB_Error_Code hook (EB_Book* book,  EB_Appendix* appendix,  \
                            void* container,  EB_Hook_Code code,  \
                            int argc, const unsigned *argv)
extern "C" {
HOOK_PROTOTYPE(hook_font);
HOOK_PROTOTYPE(hook_newline);
HOOK_PROTOTYPE(hook_tags);
HOOK_PROTOTYPE(hook_multimedia);
    
HOOK_PROTOTYPE(heading_hook_font);
HOOK_PROTOTYPE(heading_hook_newline);
HOOK_PROTOTYPE(heading_hook_tags);
}
// Hook Table
static EB_Hook text_hooks[] = {
    /* Text Hooks */
    {EB_HOOK_NARROW_JISX0208, eb_hook_euc_to_ascii},
    {EB_HOOK_NARROW_FONT,     hook_font},
    {EB_HOOK_WIDE_FONT,       hook_font},
    {EB_HOOK_NEWLINE,         hook_newline},
    {EB_HOOK_STOP_CODE,       eb_hook_stop_code},
    {EB_HOOK_SET_INDENT,      hook_tags},
    {EB_HOOK_GB2312,          hook_tags},

    {EB_HOOK_BEGIN_REFERENCE, hook_tags},
    {EB_HOOK_END_REFERENCE,   hook_tags},

    {EB_HOOK_BEGIN_EMPHASIS,  hook_tags},
    {EB_HOOK_END_EMPHASIS,    hook_tags},
    {EB_HOOK_BEGIN_SUPERSCRIPT, hook_tags},
    {EB_HOOK_END_SUPERSCRIPT, hook_tags},
    {EB_HOOK_BEGIN_SUBSCRIPT, hook_tags},
    {EB_HOOK_END_SUBSCRIPT, hook_tags},

    {EB_HOOK_BEGIN_WAVE,      hook_multimedia},
    {EB_HOOK_END_WAVE,        hook_multimedia},

    {EB_HOOK_NULL, EMPTY},
    
};
static EB_Hook heading_text_hooks[] = {
    {EB_HOOK_NARROW_JISX0208, eb_hook_euc_to_ascii},
    {EB_HOOK_NARROW_FONT,     heading_hook_font},
    {EB_HOOK_WIDE_FONT,       heading_hook_font},
    {EB_HOOK_NEWLINE,         heading_hook_newline},
    {EB_HOOK_STOP_CODE,       eb_hook_stop_code},
    {EB_HOOK_NULL, EMPTY},
};


static int g_font_size = EB_SIZE_NARROW_FONT_16;


EB_Font_Code                g_fontlist[EB_MAX_FONTS];

#define  HTML_TABLE_START               "<table cellspacing=\"0\" cellpadding=\"0\"><tr>"
#define  HTML_TABLE_END                 "</tr></table>"
#define  HTML_TABLE_DATA_START          "<td nowrap>"
#define  HTML_TABLE_DATA_END            "</td>"
#define  HTML_IMG                       "<img "

const size_t HTML_TABLE_START_LEN       = strlen(HTML_TABLE_START);
const size_t HTML_TABLE_END_LEN         = strlen(HTML_TABLE_END);
const size_t HTML_TABLE_DATA_START_LEN  = strlen(HTML_TABLE_DATA_START);
const size_t HTML_TABLE_DATA_END_LEN    = strlen(HTML_TABLE_DATA_END);
const size_t HTML_IMG_LEN               = strlen(HTML_IMG);

bool save_font_image(const char* filename, const char* imagedata, size_t imagesize)
{
    FILE *fp = fopen(filename, "wb");

    if (fp == EMPTY) {
        cerr << __FILE__ << ":" << __LINE__ << ":"
             << "Cannot open the font file, "
             << strerror(errno) << filename
             << endl;
        return false;
    }
    if (fwrite(imagedata, 1, imagesize, fp) < imagesize) {
        cerr << __FILE__ << ":" << __LINE__ << ":"
             << "Error fwrite :" << filename << endl;
        return false;
    }

    fclose(fp);

    return true;
}
void fontpath(string& fontfilename, const char* fonttype, int c)
{
    char filetitle[4+1];
    sprintf(filetitle, "%04x", c);
    fontfilename = *g_font_path+DIRECTORY_SEPARATOR;
    fontfilename += fonttype;
    fontfilename += DIRECTORY_SEPARATOR;
    fontfilename += string(filetitle);
    fontfilename += ".xbm";
}
bool isfile(const string& filename)
{
    struct stat sb;
    if (stat(filename.c_str(), &sb) != RC_ERR &&
        S_ISREG(sb.st_mode)){
        return true;
    }
    return false;
}
bool createNarrowFontFile(EB_Book* book,
                          const string& fontfilename,
                          unsigned ch)
{
    char bitmap[EB_SIZE_NARROW_FONT_48];
    EB_Font_Code *list = g_fontlist;
    while (!eb_have_font(book, *list)) {
        list++;
        if (list-g_fontlist >= EB_MAX_FONTS) {
            return false;
        }
    }
    //eb_set_font(book, BASE_FONT_TYPE);
    EB_Error_Code rc;
    if ((rc = eb_narrow_font_character_bitmap(book, ch, bitmap)) != EB_SUCCESS) {
        cerr << __FILE__ << ":" << __LINE__ << ":"
             << "eb_narrow_font_character_bitmap() : "
             << eb_error_message(rc)
             << " : 0x" << hex << ch << endl;
        return false;
    }
                
    char imagedata[EB_SIZE_FONT_IMAGE];
    size_t imagelength;
    eb_bitmap_to_xbm(bitmap, 
                     BASE_FONT_SIZE/2, BASE_FONT_SIZE,
                     imagedata, &imagelength);
    //eb_unset_font(book);
    save_font_image(fontfilename.c_str(), imagedata, imagelength);
    return true;
}
bool createWideFontFile(EB_Book* book,
                        const string& fontfilename,
                        unsigned ch)
{
    char bitmap[EB_SIZE_WIDE_FONT_48];

    EB_Font_Code *list = g_fontlist;
    while (!eb_have_font(book, *list)) {
        list++;
        if (list-g_fontlist >= EB_MAX_FONTS) {
            return false;
        }
    }
    //eb_set_font(book, BASE_FONT_TYPE);
    EB_Error_Code rc = eb_wide_font_character_bitmap(book, ch, bitmap);
    if (rc != EB_SUCCESS) {
        cerr << __FILE__ << ":" << __LINE__ << ":"
             << "eb_wide_font_character_bitmap : "
             << eb_error_message(rc)
             << " : 0x" << hex << ch << endl;
        return false;
    }
    char imagedata[EB_SIZE_FONT_IMAGE];
    size_t imagelength;
    eb_bitmap_to_xbm(bitmap, 
                     BASE_FONT_SIZE, BASE_FONT_SIZE,
                     imagedata, &imagelength);
    //eb_unset_font(book);
    
    save_font_image(fontfilename.c_str(), imagedata, imagelength);

    return true;
}
extern "C" {
EB_Error_Code hook_font(EB_Book* book,
                        EB_Appendix* appendix,
                        void* /*container*/,
                        EB_Hook_Code code,
                        int, const unsigned *argv)
{
    EB_Error_Code rc;
    char alt_text[EB_MAX_ALTERNATION_TEXT_LENGTH + 1];
    switch (code) {
    case EB_HOOK_NARROW_FONT:
        rc = eb_narrow_alt_character_text(appendix, argv[0], alt_text);
        if (rc != EB_SUCCESS) {
            string fontfilename;
            fontpath(fontfilename, "narrow", argv[0]);
            strstream buff;
            buff <<
                HTML_TABLE_DATA_END HTML_TABLE_DATA_START
                HTML_IMG "src=\"file:"
                 << fontfilename
                 << "\" alt=\"GAIJI="
                 << hex << setfill('0') << setw(4)
                 << argv[0] << "\" "
                 << "width=\""
                 << dec << g_font_size/2
                 << "\" height=\""
                 << dec << g_font_size << "\">"
                 << HTML_TABLE_DATA_END HTML_TABLE_DATA_START
                 << ends;
            rc = eb_write_text_string(book, buff.str()); 
            if (rc != EB_SUCCESS) {
                cerr << __FILE__ << ":" << __LINE__ << ":"
                     << eb_error_message(rc) << '(' << rc << ')' << endl;
                return rc;
            }
            if (!isfile(fontfilename)) {
                createNarrowFontFile(book, fontfilename, argv[0]);
            }
        }
        break;
    case EB_HOOK_WIDE_FONT:
        rc = eb_wide_alt_character_text(appendix, argv[0],  alt_text);
        if (rc != EB_SUCCESS) {
            string fontfilename;
            fontpath(fontfilename, "wide", argv[0]);
            strstream buff;
            buff << HTML_TABLE_DATA_END HTML_TABLE_DATA_START
                 << HTML_IMG "src=\"file:"<< fontfilename
                 << "\" alt=\"GAIJI="
                 << hex << setfill('0') << setw(4)
                 << argv[0] << "\" "
                 << "width=\""
                 << dec << g_font_size
                 << "\" height=\""
                 << dec << g_font_size << "\">"
                 << HTML_TABLE_DATA_END HTML_TABLE_DATA_START
                 << ends;
            rc = eb_write_text_string(book, buff.str());
            if (rc != EB_SUCCESS) {
                cerr << __FILE__ << ":" << __LINE__ << ":"
                     << eb_error_message(rc) << '(' << rc << ')' << endl;
                return rc;
            }
            if (!isfile(fontfilename)) {
                createWideFontFile(book, fontfilename, argv[0]);
            }
        }
        break;
    }
    return EB_SUCCESS;
}
EB_Error_Code heading_hook_font(EB_Book* book,
                                EB_Appendix* appendix,
                                void* /*container*/,
                                EB_Hook_Code code,
                                int , const unsigned *argv)
{
    EB_Error_Code rc = EB_SUCCESS;
    char alt_text[EB_MAX_ALTERNATION_TEXT_LENGTH + 1];
    switch (code) {
    case EB_HOOK_NARROW_FONT:
        rc = eb_narrow_alt_character_text(appendix, argv[0], alt_text);
        if (rc != EB_SUCCESS) {
            strcpy(alt_text, "?");
        }
        rc = eb_write_text_string(book, alt_text);
        break;
    case EB_HOOK_WIDE_FONT:
        rc = eb_wide_alt_character_text(appendix, argv[0], alt_text);
        if (rc != EB_SUCCESS) {
            strcpy(alt_text, "??");
        }
        rc = eb_write_text_string(book, alt_text);
        break;
    }
    return rc;
}

EB_Error_Code hook_newline(EB_Book* book,
                           EB_Appendix*,
                           void* /* container */,
                           EB_Hook_Code,
                           int, const unsigned*)
{
    const char* str =
        HTML_TABLE_DATA_END
        HTML_TABLE_END
        HTML_TABLE_START
        HTML_TABLE_DATA_START;
    return eb_write_text_string(book, str);
}
EB_Error_Code heading_hook_newline(EB_Book* book,
                                   EB_Appendix*,
                                   void* /*container*/,
                                   EB_Hook_Code,
                                   int, const unsigned*)
{
    return eb_write_text_string(book, "\n");
}
EB_Error_Code hook_tags(EB_Book* book,
              EB_Appendix*,
              void* container,
              EB_Hook_Code code,
              int, const unsigned* argv)
{
    EB_Position pos;
    strstream buff;
    
    vector<EB_Position>* pos_array = (vector<EB_Position>*)container;

    switch (code) {
    case EB_HOOK_BEGIN_REFERENCE:
        buff << "<a href=\"eb://" << INSERT_MARK << "\">";
        break;
    case EB_HOOK_END_REFERENCE:
        buff << "</a>";
        pos.page = argv[1];
        pos.offset = argv[2];
        pos_array->push_back(pos);
        break;
    case EB_HOOK_SET_INDENT:
        buff << "&nbsp;&nbsp;";
        break;
    case EB_HOOK_GB2312:
        buff << hex << argv[0] - 0xa0a0;
        cerr << "GB2312" << endl;
        break;
    case EB_HOOK_BEGIN_EMPHASIS:
        buff <<  "<strong>";
        break;
    case EB_HOOK_END_EMPHASIS:
        buff << "</strong>";
        break;
    case EB_HOOK_BEGIN_SUPERSCRIPT:
        buff << "<sup>";
        break;
    case EB_HOOK_END_SUPERSCRIPT:
        buff << "</sup>";
        break;
    case EB_HOOK_BEGIN_SUBSCRIPT:
        buff << "<sub>";
        break;
    case EB_HOOK_END_SUBSCRIPT:
        buff << "</sub>";
        break;
    default:
        cerr << "HOOK:" << code << endl;
        break;
    }
    buff << ends;
    return eb_write_text_string(book, buff.str());
}
EB_Error_Code hook_multimedia(EB_Book* book,
              EB_Appendix*,
              void* container,
              EB_Hook_Code code,
              int, const unsigned* argv)
{
    strstream buff;
    EB_Position pos;
    vector<EB_Position>* pos_array = (vector<EB_Position>*)container;
    
    switch (code) {
    case EB_HOOK_BEGIN_WAVE:
        buff << "<a href=\"ebsound://"
             << INSERT_MARK << "\">";
        break;
    case EB_HOOK_END_WAVE:
        buff << "</a>";
        pos.page = argv[1];
        pos.offset = argv[2];
        pos_array->push_back(pos);
        break;
    }

    buff << ends;
    return eb_write_text_string(book, buff.str());
}
EB_Error_Code heading_hook_tags(EB_Book*,
                                EB_Appendix*,
                                void*,
                                EB_Hook_Code,
                                int, const unsigned*)
{
    return EB_SUCCESS;
}

}

CEBookError CEBook::initialize()
{
    CEBookError error;
    EB_Error_Code rc = eb_initialize_library();
    if (rc != EB_SUCCESS) {
        cerr << __FILE__ << ":" << __LINE__ << ":"
             << eb_error_message(rc) << '(' << rc << ')' << endl;
        error.setError(rc);
    }
    return error;
}
CEBook::CEBook()
    : m_subbooks(EMPTY), m_wrapcol(40)
{
    m_font_path = new string("");
    m_dict_path = new string("");
    //m_subbook_directory = new string("");
    
    eb_initialize_book(&m_book);
    eb_initialize_appendix(&m_appendix);
    
    eb_initialize_hookset(&m_text_hookset);
    eb_set_hooks(&m_text_hookset, text_hooks);

    eb_initialize_hookset(&m_heading_text_hookset);
    eb_set_hooks(&m_heading_text_hookset, heading_text_hooks);
}
CEBook::CEBook(const char* dict_path, const char* font_path)
{
    setDictPath(dict_path);
    setFontPath(font_path);
}
CEBook::~CEBook()
{
    delete m_subbooks;
    delete m_font_path;
    delete m_dict_path;
    //delete m_subbook_directory;
    
    eb_finalize_hookset(&m_text_hookset);
    eb_finalize_hookset(&m_heading_text_hookset);
    eb_finalize_book(&m_book);
}
void CEBook::setSubbookInfoList()
{
    EB_Subbook_Code sub_books[EB_MAX_SUBBOOKS];
    int count;
    EB_Error_Code rc = eb_subbook_list(&m_book, sub_books, &count);
    if (rc != EB_SUCCESS) {
        cerr << __FILE__ << ":" << __LINE__ << ":"
             << eb_error_message(rc) << '(' << rc << ')' << endl;
        m_error.setError(rc);
        return;
    }
    m_subbooks = new SubbookInfoList();
    int i = 0;
    while (i < count) {
        SubbookInfo info;
        char title[EB_MAX_TITLE_LENGTH];
        rc = eb_subbook_title2(&m_book, sub_books[i], title);
        info.title = title;
        char directory[EB_MAX_DIRECTORY_NAME_LENGTH];
        rc = eb_subbook_directory2(&m_book, sub_books[i], directory);
        info.directory = directory;
        info.code = sub_books[i];
        m_subbooks->push_back(info);
        i++;
    }
    m_cur_subbook = m_subbooks->begin();

}
int CEBook::getSubbook()
{
    if (m_subbooks != EMPTY && !m_subbooks->empty())
        return m_cur_subbook-m_subbooks->begin();
    else
        return RC_ERR;
}
void CEBook::setSubbook(SubbookInfoListIterator it)
{
    EB_Error_Code rc = eb_set_subbook(&m_book, (*it).code);
    if (rc != EB_SUCCESS) {
        cerr << __FILE__ << ":" << __LINE__ << ":"
             << eb_error_message(rc) << '(' << rc << ')' << endl;
        m_error.setError(rc);
        return;
    }
    m_cur_subbook = it;
    int font_count;
    eb_font_list(&m_book, g_fontlist, &font_count);
    eb_set_font(&m_book, BASE_FONT_TYPE);
    eb_set_appendix_subbook(&m_appendix, (*m_cur_subbook).code);
}
void CEBook::unsetSubbook()
{
    eb_unset_appendix_subbook(&m_appendix);
    eb_unset_font(&m_book);
    eb_unset_subbook(&m_book);
    m_cur_subbook = m_subbooks->end();
}
const char* CEBook::getSubbookDirectory()
{
    return (*m_cur_subbook).directory.c_str();
}
void CEBook::getHeading(string& heading, EB_Position* pos)
{
    heading = "";
    char buff[BUFSIZE];
    buff[sizeof (buff)-1] = '\0';

    eb_seek_text(&m_book, pos);
    
    g_font_path = m_font_path;
    EB_Error_Code rc;
    ssize_t length;
    
    rc = eb_read_text(&m_book, &m_appendix,
                      &m_heading_text_hookset,
                      NULL,
                      sizeof (buff)-1, buff, &length);
    while (rc == EB_SUCCESS && length > 0) {
        char* nl;
        if ((nl = strchr(buff, '\n')) != EMPTY) {
            *nl = '\0';
            heading += buff;
            break;
        } else {
            heading += buff;
        }
    }
}
void CEBook::getContent(string& content, EB_Position* pos)
{
    content = HTML_TABLE_START HTML_TABLE_DATA_START;
    
    char buff[BUFSIZE];
    buff[sizeof (buff)-1] = '\0';
    string body = "";
    
    eb_seek_text(&m_book, pos);
    g_font_path = m_font_path;
    ssize_t length;
    EB_Error_Code rc;
    for (;;) {
        rc = eb_read_text(&m_book, &m_appendix, &m_text_hookset,
                          (void *)&m_pos_array,
                          sizeof (buff)-1, buff, &length);
        if (rc == EB_SUCCESS && length > 0) {
            *(buff+length) = '\0';
            body += buff;
        } else {
            break;
        }
    }
    // Insert Link Address
    vector<EB_Position>::iterator it = m_pos_array.begin();
    string::iterator p = body.begin();
    while (p != body.end()) {
        if (*p != INSERT_MARK) {
            content += *p;
        } else {
            char page_offset[32];
            sprintf(page_offset, "%d:%d", (*it).page, (*it).offset);
            content += page_offset;
            it++;
        }
        p++;
    }
    m_pos_array.clear();
    
    content += HTML_TABLE_DATA_END HTML_TABLE_END;
}
void CEBook::getContent(string& content, int page, int offset)
{
    EB_Position pos;
    pos.page = (int)page;
    pos.offset = (int)offset;
    getContent(content, &pos);
}

struct less_EB_Position :
    binary_function <EB_Position, EB_Position, bool>
{
    bool operator()(const EB_Position& a, const EB_Position& b) const
    {
        if (a.page != b.page) {
            return a.page < b.page;
        } else {
            return a.offset < b.offset;
        }
    }
};
static void parseKeyWords(const char* word, string words[])
{
    int index = 0;
    do {
        words[index] = "";
        while ((*word == ' ' || *word == '\t') && *word != '\0') {
            word++;
        }
        while (*word != ' ' && *word != '\t' && *word != '\0') {
            words[index] += *word++;
        }
        index++;
    } while (*word != '\0' && index < EB_MAX_KEYWORDS+1);
    words[index] = "";
}
void CEBook::lookupWord(CEBook::LookupClient* client,
                        CEBook::LookupResult& result,
                        const char* word,
                        CEBook::WAY_TO_SEARCH way)
{
    int (*have_search)(EB_Book*);
    int (*search_word)(EB_Book*, const char*);
    switch (way) {
    case EXACT:
        have_search = eb_have_exactword_search;
        search_word = eb_search_exactword;
        break;
    case FRONT:
        have_search = eb_have_word_search;
        search_word = eb_search_word;
        break;
    case TAIL:
        have_search = eb_have_endword_search;
        search_word = eb_search_endword;
        break;
    default:
        have_search = eb_have_exactword_search;
        search_word = eb_search_exactword;
        break;
    }
    string words[EB_MAX_KEYWORDS+1];
    const char* keywords[EB_MAX_KEYWORDS+1];
    parseKeyWords(word, words);
    int index = 0;
    while (index <= EB_MAX_KEYWORDS+1 &&
           words[index] != "") {
        keywords[index] = words[index].c_str();
        index++;
    }
    keywords[index] = EMPTY;

    EB_Error_Code rc = EB_SUCCESS;
    if (index == 1) {
        if (!(*have_search)(&m_book)) {
            m_error.setError(rc);
            return;
        }
        if ((rc = (*search_word)(&m_book, word)) != EB_SUCCESS) {
            cerr << __FILE__ << ":" << __LINE__ << ":"
                 << eb_error_message(rc) << '(' << rc << ')' << endl;
            m_error.setError(rc);
            return;
        }
    } else {
        if (!eb_have_keyword_search(&m_book)) {
            m_error.setError(EBEX_ERR_NO_KEYWORD_SEARCH);
            return ;
        }
        if ((rc = eb_search_keyword(&m_book, keywords)) != EB_SUCCESS) {
            cerr << __FILE__ << ":" << __LINE__ << ":"
                 << eb_error_message(rc) << '(' << rc << ')' << endl;
            m_error.setError(rc);
            return;
        }
    }
    
    EB_Hit hits[MAX_HITS];
    map<EB_Position, string, less_EB_Position> prevresult;
    int hitcount, totalcount = 0;
    string heading, content;
    int counter_showprogress = 1;
    
    while (eb_hit_list(&m_book, MAX_HITS, hits, &hitcount) == EB_SUCCESS &&
           hitcount > 0) {
        for (int i = 0; i < hitcount; i++) {
            getHeading(heading, &hits[i].heading);
            // Ignore a hit entry which equals to
            // a previous hit entry.
            if (!prevresult.empty() &&
                prevresult.count(hits[i].text) &&
                prevresult[hits[i].text] == heading) {
                continue;
            }
            getContent(content, &hits[i].text);
            client->enterCriticalSection(&result);
            result.push_back(LookupResultElement(heading, content));
            client->leaveCriticalSection(&result);
            if (client != EMPTY) {
                if (counter_showprogress >= SHOW_PROGRESS_COUNT
                    || i == hitcount-1) {
                    client->showProgress(totalcount-counter_showprogress+1,
                                         counter_showprogress,
                                         (i*100)/hitcount);
                    counter_showprogress = 0;
                }
            }
            prevresult[hits[i].text] = heading;
            
            if (client->isStopped()) {
                if (counter_showprogress != 0) {
                    client->showProgress(totalcount-counter_showprogress+1,
                                         counter_showprogress,
                                         (i*100)/hitcount);
                }
                goto end;
            }
            totalcount++;
            counter_showprogress++;
        }
    }
end:
    return;
}
void CEBook::getCopyright(string& copyright)
{
    EB_Position pos;
    EB_Error_Code rc;
    if ((rc = eb_copyright(&m_book, &pos)) != EB_SUCCESS) {
        cerr << __FILE__ << ":" << __LINE__ << ":"
             << eb_error_message(rc) << '(' << rc << ')' << endl;
        copyright = "This dictionry has no copyright information.";
    } else {
        getContent(copyright, pos.page, pos.offset);
    }
    
}
void CEBook::getDictMenu(string& menu)
{
    EB_Position pos;
    if (eb_have_menu(&m_book)) {
        eb_menu(&m_book, &pos);
        getContent(menu, pos.page, pos.offset);
    } else {
        menu = "This dictinary has no menu.";
    }
}
void CEBook::setDictPath(const char* dict_path)
{
    EB_Error_Code rc = eb_bind(&m_book, dict_path);
    if (rc != EB_SUCCESS) {
        cerr << __FILE__ << ":" << __LINE__ << ":"
             << eb_error_message(rc) << '(' << rc << ')' << endl;
        m_error.setError(rc);
    }
    *m_dict_path = dict_path;
    setSubbookInfoList();
}
void CEBook::setFontPath(const char* font_path)
{
    *m_font_path = font_path;
}
const char* CEBook::getFontPath()
{
    return m_font_path->c_str();
}
int CEBook::getFontSize()
{
    return g_font_size;
}
void CEBook::setFontSize(int font_size)
{
    g_font_size = font_size;
}
bool CEBook::isAvailableDictPath(const char* path)
{
    EB_Book book;
    eb_initialize_book(&book);
    EB_Error_Code rc;
    if ((rc = eb_bind(&book, path)) != EB_SUCCESS) {
        cerr << __FILE__ << ":" << __LINE__ << ":"
             << eb_error_message(rc) << '(' << rc << ')' << endl;
        eb_finalize_book(&book);
        return false;
    }
    eb_finalize_book(&book);
    return true;
}
const char*         CEBook::getRowStart()
{
    return HTML_TABLE_START HTML_TABLE_DATA_START;
}
const char*         CEBook::getRowEnd()
{
    return HTML_TABLE_DATA_END HTML_TABLE_END;
}
