/*
 * $Id: kebookwidget.cpp,v 1.5 2000/12/23 14:51:36 hige Exp $
 *
 * Copyright (C) 1999 SUZUKI, Mitsuhiro <hige@higesoft.rim.or.jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include "kebook.h"
#include "kebookapp.h"
#include "kebookwidget.h"
#include "findworddlg.h"

//#include <kclipboard.h>
#include <kcombobox.h>
#include <kaccel.h>
#include <kkeydialog.h>
#include <klocale.h>
#include <khtml_part.h>
#include <kmenubar.h>
#include <kmessagebox.h>

#include <qpushbutton.h>
#include <qlistbox.h>
#include <qkeycode.h>
#include <qregexp.h>
#include <qwidget.h>
#include <qevent.h>

#include <pthread.h>
#include <string>

#include "changecolordlg.h"

const int RESULT_FONT_SIZE      =  14;
const int GUIPARTS_MARGIN       = 1;
const int GUIPARTS_FONTSIZE     = 12;
const int LB_MATCH_WORDS_WIDTH  = 300;
const int CB_LOOKUP_WIDTH       = 100;
const int CB_LOOKUP_HEIGHT      = 30;
const int BTN_LOOKUP_WIDTH      = 75;
const int BTN_LOOKUP_HEIGHT     = 30;

KEBookWidget::KEBookWidget(QWidget *parent, const char *name)
    : QWidget(parent, name),
      m_resultpart_changed(false)
{
    // Create GUI Parts
    createLookupComboBox();
    createLookupButton();
    createMatchWordsListBox();
    createResultPart();

    // Set Default Focus
    m_cb_lookup->setFocus();

    // Connect signal
    connect(parent, SIGNAL(sigSelectedSubbook()),
            SLOT(enableButtons()));
    connect(parent, SIGNAL(sigReleaseSubbook()),
            SLOT(disableButtons()));
    connect(parent, SIGNAL(sigSuspendGUIParts()),
            SLOT(suspendGUIParts()));
    connect(parent, SIGNAL(sigResumeGUIParts()),
            SLOT(resumeGUIParts()));
    connect(parent, SIGNAL(sigAddMatchWordsList(int, int)),
            SLOT(addMatchWordsList(int, int)));
    
    // Create Keyboard Accelerator
    createAcceleration();
    
    m_find_text_page_dlg_pos.setX(parent->x());
    m_find_text_page_dlg_pos.setY(parent->y());

    m_finddlg = new KFindWordDlg(this);
    connect(m_finddlg, SIGNAL(sigSearch()),
            SLOT(slotFindNextWordInPage()));
    connect(m_finddlg, SIGNAL(sigSearchDone()),
            SLOT(closeFindWordInPageDlg()));
    connect(kapp, SIGNAL(kdisplayPaletteChanged()),
            SLOT(showCurrentResult()));
}
KEBookWidget::~KEBookWidget()
{
    m_accel->writeSettings();
}
void KEBookWidget::createAcceleration()
{
    m_accel = new KAccel(this);
    // ResultPart's page up/down (Default: PageUp/ PageDown Key)
    m_accel->insertItem(i18n("Page up"), "Page Up", Key_PageUp);
    m_accel->connectItem("Page Up", m_rp_result, SLOT(slotPageUp()));
    m_accel->insertItem(i18n("Page down"), "Page Down", Key_PageDown);
    m_accel->connectItem("Page Down", m_rp_result, SLOT(slotPageDown()));

    // Scroll ResultPart (Default: Ctrl+Key_Up/Key_Down)
    m_accel->insertItem(i18n("Scroll up"), "Scroll Up", CTRL+Key_Up);
    m_accel->connectItem("Scroll Up", m_rp_result, SLOT(slotScrollVertUp()));
    m_accel->insertItem(i18n("Scroll down"), "Scroll Down", CTRL+Key_Down);
    m_accel->connectItem("Scroll Down", m_rp_result, SLOT(slotScrollVertDown()));

    // Scroll MatchWordsListBox (Default: Shift+Key_Up/Key_Down)
    m_accel->insertItem(i18n("Scroll up match words list box"),
                        "Scroll Up Match Words List Box", SHIFT+Key_Up);
    m_accel->connectItem("Scroll Up Match Words List Box",
                         this, SLOT(scrollUpMatchWordsListBox()));
    m_accel->insertItem(i18n("Scroll down match words list box"),
                        "Scroll Down Match Words List Box", SHIFT+Key_Down);
    m_accel->connectItem("Scroll Down Match Words List Box",
                        this, SLOT(scrollDownMatchWordsListBox()));
    
    // Escape Looking up (Default: Escape Key)
    m_accel->insertItem(i18n("Escape"), "Escape", Key_Escape);
    m_accel->connectItem("Escape", getApp(), SLOT(stopLookingUp()));

    // Move in history (Default: ALT+Left/Right & Ctrl+P/N)
    m_accel->insertItem(i18n("Back"), "Page Back(1)", ALT+Key_Left);
    m_accel->connectItem("Page Back(1)", this, SLOT(onBack()));
    m_accel->insertItem(i18n("Forward"), "Page Forward(1)", ALT+Key_Right);
    m_accel->connectItem("Page Forward(1)", this, SLOT(onForward()));
    
    m_accel->insertItem(i18n("Back"), "Page Back(2)", CTRL+Key_P);
    m_accel->connectItem("Page Back(2)", this, SLOT(onBack()));
    m_accel->insertItem(i18n("Forward"), "Page Forward(2)", CTRL+Key_N);
    m_accel->connectItem("Page Forward(2)", this, SLOT(onForward()));

    // Start Looking up (Default: Ctrl+M)
    m_accel->insertItem(i18n("Look up"), "Look up", Key_M+CTRL);
    m_accel->connectItem("Look up", this, SLOT(onClickLookup()));
    m_accel->readSettings();
}
void KEBookWidget::createLookupComboBox()
{
    m_cb_lookup = new KComboBox(true, this, "lookup combo box");
    CHECK_PTR(m_cb_lookup);
    m_cb_lookup->setFont(QFont("Times", GUIPARTS_FONTSIZE, QFont::Normal));
    m_cb_lookup->setInsertionPolicy(KComboBox::NoInsertion);
    m_cb_lookup->setGeometry(0, 0, CB_LOOKUP_WIDTH, CB_LOOKUP_HEIGHT);
    m_cb_lookup->insertItem("", 0);
}
void KEBookWidget::createLookupButton()
{
    m_btn_lookup = new QPushButton(i18n("Look up"), this, "lookup button");
    CHECK_PTR(m_btn_lookup);
    m_btn_lookup->setFont(QFont("Times", GUIPARTS_FONTSIZE, QFont::Bold));
    m_btn_lookup->setGeometry(m_cb_lookup->width()+GUIPARTS_MARGIN, 0,
                              BTN_LOOKUP_WIDTH, BTN_LOOKUP_HEIGHT);
    connect(m_btn_lookup, SIGNAL(clicked()), SLOT(onClickLookup()));
    m_btn_lookup->setEnabled(false);
}
void KEBookWidget::createMatchWordsListBox()
{
    m_lb_match_words = new QListBox(this, "match words list box");
    CHECK_PTR(m_lb_match_words);
    m_lb_match_words->setFont(QFont("Times", GUIPARTS_FONTSIZE, QFont::Normal));
    m_lb_match_words->move(0, m_cb_lookup->y()
                           +m_cb_lookup->height()+GUIPARTS_MARGIN);
    m_lb_match_words->resize(m_btn_lookup->width()
                             +m_cb_lookup->width()+GUIPARTS_MARGIN,
                             LB_MATCH_WORDS_WIDTH);
    connect(m_lb_match_words, SIGNAL(highlighted(int)),
            SLOT(onChangeOtherMatchWord(int)));
}
void KEBookWidget::createResultPart()
{
    m_rp_result = new KResultPart(KEBookApp::getFontSize(),
                                  this, "result part");
    CHECK_PTR(m_rp_result);
    
    // Font Size
    QValueList<int> newFontSizes;
    for (int i = 0; i < 7; i++)
        newFontSizes.append(RESULT_FONT_SIZE);

    m_rp_result->setFontSizes(newFontSizes);
    //htmlview->setFocusPolicy((QWidget::FocusPolicy)(TabFocus | ClickFocus));
    m_rp_result->view()->move(2+m_lb_match_words->x()+m_lb_match_words->width(), 0);
    m_rp_result->view()->resize(400, m_lb_match_words->height()+
                                m_lb_match_words->y());
    connect(m_rp_result, SIGNAL(sigClickLink(int, long)),
            SLOT(showPage(int, long)));
    string startmsg =
        (const char*)i18n("<h3>Dictionary isn't selected.</h3>"
                          "<p>You must do first:</p>"
                          "<dl>"
                          "<dt><p>(1)Choose the directory to contain the dictionary.</p>"
                          "<dd><p>[Options]->[Set a Directory to contain the dictionary...]</p>"
                          "<dt><p>(2)Choose one dictionary in the directory.</p>"
                          "<dd><p>[Options]->[Select a Dictionary...]</p>"
                          "</dl>").local8Bit();
    m_rp_result->printContent(string("start"), startmsg, "<br>", "");
}
void KEBookWidget::printResult(const string& title, const string& content)
{
    m_rp_result->printContent(title, content,
                              getApp()->getRowStart(),
                              getApp()->getRowEnd());
    getApp()->pushResult(title, content);
    KMenuBar* menubar = parentWidget()->menuBar();
    KToolBar* toolbar = parentWidget()->toolBar();
    if (getApp()->isHistoryBegin()) {
        menubar->setItemEnabled((int)ID_MENUITEM_BACK, false);
        toolbar->setItemEnabled((int)ID_TBBTN_BACK, false);
    } else {
        menubar->setItemEnabled(ID_MENUITEM_BACK, true);
        toolbar->setItemEnabled(ID_TBBTN_BACK, true);
    }
    if (getApp()->isHistoryEnd()) {
        menubar->setItemEnabled(ID_MENUITEM_FORWARD, false);
        toolbar->setItemEnabled(ID_TBBTN_FORWARD, false);
    } else {
        menubar->setItemEnabled(ID_MENUITEM_FORWARD, true);
        toolbar->setItemEnabled(ID_TBBTN_FORWARD, true);
    }
    if (getApp()->isLookingupThreadAlive()) {
        m_resultpart_changed = true;
    }
}
void KEBookWidget::addMatchWordsList(int i, int count)
{
    if (getApp()->getMatchWordsCount()) {
        for (int j = i; j < i+count; j++) {
            const QString& item = QString::fromLocal8Bit(getApp()->getMatchWord(j).c_str());
            m_lb_match_words->insertItem(item);
        }
    }
}
void KEBookWidget::finishLookup()
{
    if (getApp()->getMatchWordsCount() > 0) {
        if (!m_resultpart_changed) {
            printResult(getApp()->getMatchWord(0),
                        getApp()->getLookupResult(0));
            m_resultpart_changed = false;
        }
    } else if (!getApp()->isLookingupError()) {
        QString msg = "<h2>";
        msg += i18n("Not found");
        msg += "</h2>";
        msg += "<p>";
        msg += i18n("Not found such a word.");
        msg += "</p>";
        printResult((const char*)i18n("Not found").local8Bit(),
                    (const char*)msg.local8Bit());
    } else {
        QString msg = "<h2>";
        msg += i18n("Error");
        msg += "</h2>";
        msg += QString::fromLocal8Bit(("<p>"+getApp()->getEBErrorMessage()+
                                       "</p>").c_str());
        printResult((const char*)i18n("Error").local8Bit(),
                    (const char*)msg.local8Bit());
    }
}
void KEBookWidget::onClickLookup()
{
    QCString curlocal = m_cb_lookup->currentText().local8Bit();
    if (curlocal.isNull() || curlocal.isEmpty()
        || !m_btn_lookup->isEnabled()) {
        return;
    }
    m_resultpart_changed = false;
    parentWidget()->preLookupWord(curlocal);
    clearMatchWordsList();
    m_cur_lookupword = (const char*)curlocal;
    getApp()->start((void *)m_cur_lookupword.c_str());
    
    int j;
    for (j = 0; j < m_cb_lookup->count(); j++) {
        if (m_cb_lookup->currentText() == m_cb_lookup->text(j))
            break;
    }
    if (j == m_cb_lookup->count()) {
       m_cb_lookup->insertItem(m_cb_lookup->currentText(), 0);
    }
    m_cb_lookup->clearEdit();
}
void KEBookWidget::onChangeOtherMatchWord(int i)
{
    printResult(getApp()->getMatchWord(i),
                getApp()->getLookupResult(i));
    if (getApp()->isLookingupThreadAlive()) {
        m_resultpart_changed = true;
    }
}
void KEBookWidget::showPage(int frame, long offset)
{
    string content;
    getApp()->getContent(content, frame, offset);
    if (!content.empty()) {
        printResult(string("link"), content);
    }
    
}
void KEBookWidget::resizeEvent(QResizeEvent* event)
{
    QWidget::resizeEvent(event);
    KHTMLView* view = m_rp_result->view();
    view->resize(event->size().width()-view->x(),
                 event->size().height()-view->y());
    showCurrentResult();
    m_lb_match_words->resize(event->size().width()-view->width()-5,
                             event->size().height()-m_lb_match_words->y());
}
void KEBookWidget::showCurrentResult()
{
    string title, content;
    if (getApp()->getCurrentResult(title, content))
        m_rp_result->printContent(title, content,
                                  getApp()->getRowStart(),
                                  getApp()->getRowEnd());

}
void KEBookWidget::keyPressEvent(QKeyEvent* e)
{
    switch (e->key()) {
    case Key_Enter:
    case Key_Return:
        onClickLookup();
        break;
    }
    QWidget::keyPressEvent(e);
}
void KEBookWidget::setLookupText(const char* str)
{
    if (str) {
        const char* strend = str;
        string buff;
        while (*strend != '\n' && *strend != '\0') {
            buff += *strend++;
        }
        m_cb_lookup->setEditText(buff.c_str());
    }
}
void KEBookWidget::onBack()
{
    if (!getApp()->isHistoryBegin()) {
        string title, content;
        getApp()->movePrev(title, content);
        KToolBar* toolbar = parentWidget()->toolBar();
        KMenuBar* menubar = parentWidget()->menuBar();
        if (getApp()->isHistoryBegin()) {
            menubar->setItemEnabled(ID_MENUITEM_BACK, false);
            toolbar->setItemEnabled(ID_TBBTN_BACK, false);
        }
        menubar->setItemEnabled(ID_MENUITEM_FORWARD, true);
        toolbar->setItemEnabled(ID_TBBTN_FORWARD, true);

        m_rp_result->printContent(title, content,
                                  getApp()->getRowStart(),
                                  getApp()->getRowEnd());
    }
}
void KEBookWidget::onForward()
{
    if (!getApp()->isHistoryEnd()) {
        string title, content;
        getApp()->moveNext(title, content);
        KToolBar* toolbar = parentWidget()->toolBar();
        KMenuBar* menubar = parentWidget()->menuBar();
        if (getApp()->isHistoryEnd()){
            menubar->setItemEnabled(ID_MENUITEM_FORWARD, false);
            toolbar->setItemEnabled(ID_TBBTN_FORWARD, false);
        }
        menubar->setItemEnabled(ID_MENUITEM_BACK, true);
        toolbar->setItemEnabled(ID_TBBTN_BACK, true);

        m_rp_result->printContent(title, content,
                                  getApp()->getRowStart(),
                                  getApp()->getRowEnd());
    }
}
void KEBookWidget::enableButtons(bool enable)
{
    m_btn_lookup->setEnabled(enable);
    KToolBar* toolbar = parentWidget()->toolBar();
    toolbar->setItemEnabled(ID_TBBTN_MENU, enable);
    toolbar->setItemEnabled(ID_TBBTN_COPYRIGHT, enable);
}
void KEBookWidget::suspendGUIParts()
{
    m_btn_lookup->setEnabled(false);
    KToolBar* toolbar = parentWidget()->toolBar();
    int id;
    for (id = ID_TBBTN_BACK; id < ID_TBBTN; id++) {
        QWidget* w = toolbar->getWidget(id);
        if (w->isEnabled()) {
            m_tbbtn_enable_info.push_back(id);
            toolbar->setItemEnabled(id, false);
        }
    }
    for (id = ID_MENU_FILE; id < ID_MENU; id++) {
        parentWidget()->menuBar()->setItemEnabled(id, false);
    }
}
void KEBookWidget::resumeGUIParts()
{
    m_btn_lookup->setEnabled(true);
    KToolBar* toolbar = parentWidget()->toolBar();

    int id ;
    do  {
        id = m_tbbtn_enable_info.back();
        m_tbbtn_enable_info.pop_back();
        toolbar->setItemEnabled(id, true);
    } while (!m_tbbtn_enable_info.empty());
    
    for (id = ID_MENU_FILE; id < ID_MENU; id++) {
        parentWidget()->menuBar()->setItemEnabled(id, true);
    }
}
void KEBookWidget::slotFindWordInPage()
{
    // reset the find iterator
    m_rp_result->findTextBegin();
    m_finddlg->show();

    m_finddlg->setCaption(i18n("Find word in this page"));
    m_finddlg->move(m_find_text_page_dlg_pos);
}
void KEBookWidget::slotFindNextWordInPage()
{
    QString text = "";
    if (m_finddlg) {
        m_finddlg->getText(&text);
    }
    if (!text.isEmpty()) {
        // We have a find dialog, so use the reg exp it maintains.
        slotFindNextWordInPage(text, m_finddlg->case_sensitive());
    } else {
        // no find has been attempted yet - open the find dialog.
        slotFindWordInPage();
    }
}
void KEBookWidget::slotFindNextWordInPage(const QString& findword,
                                          bool case_sensitive)
{
    if (!m_rp_result->findTextNext(findword, true,
                                   case_sensitive)) {
        if (KMessageBox::questionYesNo(this,
                                       i18n("Continue search from the top of the page?"),
                                       i18n("Find Complete"))
            == KMessageBox::Yes) {
            m_rp_result->findTextBegin();
            slotFindNextWordInPage(findword, case_sensitive);
        } else {
            m_finddlg->hide();
        }
    }
}
void KEBookWidget::closeFindWordInPageDlg()
{
    m_find_text_page_dlg_pos = m_finddlg->pos();
    m_finddlg->hide();
}
void KEBookWidget::openKeyDlg()
{
    KKeyDialog::configureKeys(m_accel);
}
void KEBookWidget::openColorDlg()
{
    ChangeColorDlg* dlg =
        new ChangeColorDlg(m_rp_result->getColorAttribute(), this);
    if (dlg->exec()) {
        m_rp_result->setColorAttribute(dlg->getColorAttribute());
        showCurrentResult();
    }
    delete dlg;
}
void KEBookWidget::scrollUpMatchWordsListBox()
{
    QKeyEvent* ev = new QKeyEvent(QEvent::KeyPress, Key_Up, 0, 0);
    KApplication::postEvent(m_lb_match_words, ev);
}
void KEBookWidget::scrollDownMatchWordsListBox()
{
    QKeyEvent* ev = new QKeyEvent(QEvent::KeyPress, Key_Down, 0, 0);
    KApplication::postEvent(m_lb_match_words, ev);
}
#include "kebookwidget.moc"
