/*
 * $Id: kebookwidget.h,v 1.2 2000/09/22 01:06:21 hige Exp $
 *
 * Copyright (C) 1999 SUZUKI, Mitsuhiro <hige@higesoft.rim.or.jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef KEBOOKWIDGET_H 
#define KEBOOKWIDGET_H 

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kapp.h>
#include <qwidget.h>
#include <qpushbutton.h>
//#include <kclipboard.h>
#include <kcombobox.h>
#include <qlistbox.h>
#include <qcheckbox.h>

#include <string>
#include <vector>

#include "resultview.h"

class KEBook;
class KAccel;
class KEdSrch;
class KFindWordDlg;
class QColor;

/**
 * This class is the main view for KEBook.  Most non-menu, non-toolbar,
 * and non-status bar GUI code should go here.
 *
 * @short Main view
 * @author SUZUKI, Mitsuhiro <hige@higesoft.rim.or.jp>
 * @version 0.1
 */
class KEBookWidget : public QWidget
{
    Q_OBJECT
public:
    /**
     * Default constructor
     */
    KEBookWidget(QWidget *parent = 0, const char *name = 0);

    /**
     * Destructor
     */
    virtual ~KEBookWidget();
    
public:
    KEBook* parentWidget() {return (KEBook*)QWidget::parentWidget();}

    /**
     * Method for operation of GUI parts.
     */
    void printResult(const string& title, const string& content);
    void clearMatchWordsList() {
        if (m_lb_match_words->count()) m_lb_match_words->clear();
    }
    void setResultPartBackgroundColor(const QColor& newcolor)
        {m_rp_result->setBackgroundColor(newcolor);}
    void setResultPartTextColor(const QColor& newcolor)
        {m_rp_result->setTextColor(newcolor);}
    void setResultPartLinkColor(const QColor& newcolor)
        {m_rp_result->setLinkColor(newcolor);}
    
    const QColor& getResultPartBackgroundColor()
        {return m_rp_result->getColorAttribute().background;}
    const QColor& getResultPartTextColor()
        {return m_rp_result->getColorAttribute().text;}
    const QColor& getResultPartLinkColor()
        {return m_rp_result->getColorAttribute().link;}
    
public slots:
    void showPage(int frame, long offset);
    void addMatchWordsList(int i, int count);
    void slotFindWordInPage();
    void slotFindNextWordInPage();
    void slotFindNextWordInPage(const QString &regExp,
                                bool case_sensitive = true);
    void openKeyDlg();
    void openColorDlg();
    
    /**
     * Event Handlers
     */
    void finishLookup();
    
private slots:
    void onClickLookup();
    void onChangeOtherMatchWord(int);
    void resizeEvent(QResizeEvent* event);
    void enableButtons(bool enable);
    void enableButtons() {enableButtons(true);}
    void disableButtons() {enableButtons(false);}
    void onBack();
    void onForward();
    void setLookupText(const char* str);
    void suspendGUIParts();
    void resumeGUIParts();
    void closeFindWordInPageDlg();
    void scrollUpMatchWordsListBox();
    void scrollDownMatchWordsListBox();
    void showCurrentResult();
    
protected:
    virtual void keyPressEvent(QKeyEvent*e);
    
private:
    /**
     * Create GUI Parts
     */
    void createLookupComboBox();
    void createLookupButton();
    void createMatchWordsListBox();
    void createResultPart();
    void createAcceleration();
private:
    /**
     * Dialog
     */
    KFindWordDlg* m_finddlg;
    
    /**
     * GUI Parts
     */
    KComboBox*      m_cb_lookup;
    QPushButton*    m_btn_lookup;
    QListBox*       m_lb_match_words;
    KResultPart*    m_rp_result;
    KAccel*         m_accel;
    QPoint          m_find_text_page_dlg_pos;
    
    /**
     * Toolbar information for Suspend/Resume
     */
    vector<int>     m_tbbtn_enable_info;
    /**
     * Strage Currently word looking up
     */
    string          m_cur_lookupword;
    /**
     * Flags
     */
    bool            m_resultpart_changed;
};
#endif // KEBOOKWIDGET_H 
