/*
 * $Id: selectwaytosearchdlg.cpp,v 1.1.1.1 2000/05/25 06:09:58 hige Exp $
 *
 * Copyright (C) 1999 SUZUKI, Mitsuhiro <hige@higesoft.rim.or.jp>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
 
#include <kapp.h>
#include <klocale.h>

#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qpushbutton.h>
#include <qlayout.h>

#include "selectwaytosearchdlg.h"

void SelectWayToSearchDlg::createRadioButtons(QVBoxLayout* mainlayout)
{
    // ButtonGroup for Radio Button
    QButtonGroup* rbtngroup =
        new QButtonGroup(i18n("Select a way to search"),
                         this);
    CHECK_PTR(rbtngroup);
    mainlayout->addWidget(rbtngroup, 5);
    connect(rbtngroup, SIGNAL(clicked(int)),
            SLOT(clickRadioButton(int)));
    
    // Radio Button Layout
    QVBoxLayout* rbtnlayout = new QVBoxLayout(rbtngroup, 16, 1);
    CHECK_PTR(rbtnlayout);
    
    // Radio Buttons
    m_rbtn_exact = new QRadioButton(i18n("Search Exactly."),
                                    rbtngroup, "exact");
    CHECK_PTR(m_rbtn_exact);
    rbtnlayout->addWidget(m_rbtn_exact, 1);
    rbtngroup->insert(m_rbtn_exact, ebook::CEBook::EXACT);
    
    m_rbtn_front = new QRadioButton(i18n("Search words which "
                                         "contain a input word "
                                         "in the front part."),
                                    rbtngroup, "front");
    CHECK_PTR(m_rbtn_front);
    rbtnlayout->addWidget(m_rbtn_front, 1);
    rbtngroup->insert(m_rbtn_front, ebook::CEBook::FRONT);

    m_rbtn_tail = new QRadioButton(i18n("Search words which "
                                       "contain a input word "
                                       "in the tail part."),
                                   rbtngroup, "tail");
    CHECK_PTR(m_rbtn_tail);
    rbtnlayout->addWidget(m_rbtn_tail, 1);
    rbtngroup->insert(m_rbtn_tail, ebook::CEBook::TAIL);

    switch (m_way) {
    case ebook::CEBook::EXACT:
        m_rbtn_exact->setChecked(true); break;
    case ebook::CEBook::FRONT:
        m_rbtn_front->setChecked(true); break;
    case ebook::CEBook::TAIL:
        m_rbtn_tail->setChecked(true); break;
    default:
        m_rbtn_exact->setChecked(true); break;
    }
    
    rbtngroup->adjustSize();
    rbtnlayout->activate();
}
void SelectWayToSearchDlg::createPushButtons(QVBoxLayout* mainlayout)
{
    // Push Button Layout
    QHBoxLayout* btnlayout = new QHBoxLayout();
    CHECK_PTR(btnlayout);
    mainlayout->addLayout(btnlayout, 1);
    btnlayout->addSpacing(150);
    
    // Push Buttons
    QPushButton* ok = new QPushButton(i18n("OK"), this);
    CHECK_PTR(ok);
    connect(ok, SIGNAL(clicked()), SLOT(accept()));
    btnlayout->addWidget(ok);
    
    QPushButton* cancel = new QPushButton(i18n("Cancel"), this);
    CHECK_PTR(cancel);
    connect(cancel, SIGNAL(clicked()), SLOT(reject()));
    btnlayout->addWidget(cancel);
    
    btnlayout->addSpacing(150);
}
SelectWayToSearchDlg::SelectWayToSearchDlg(ebook::CEBook::WAY_TO_SEARCH way,
                         QWidget* parent, const char* name)
    : QDialog(parent, name, true)
{
    m_way = way;
    
    QVBoxLayout* mainlayout = new QVBoxLayout(this, 8, 2);
    CHECK_PTR(mainlayout);
    
    createRadioButtons(mainlayout);
    mainlayout->addSpacing(10);
    createPushButtons(mainlayout);
    
    mainlayout->activate();
    mainlayout->freeze(450, 200);
    
    setCaption(i18n("Select a way to search"));
}
void SelectWayToSearchDlg::clickRadioButton(int id)
{
    switch(id) {
        case ebook::CEBook::EXACT:
            m_way = ebook::CEBook::EXACT;
            break;
        case ebook::CEBook::FRONT:
            m_way = ebook::CEBook::FRONT;
            break;
        case ebook::CEBook::TAIL:
            m_way = ebook::CEBook::TAIL;
            break;
        default:
            m_way = ebook::CEBook::EXACT;
            break;
    }
}
#include "selectwaytosearchdlg.moc"
